@@ -1,5 +1,6 @@
 import os
 from os.path import join as pjoin
+from subprocess import check_output
 import gzip
 import numpy as np
 import nibabel as nib
@@ -182,6 +183,8 @@ def read_annot(filepath, orig_ids=False):
         Annotation id at each vertex
     ctab : numpy array
         RGBA + label id colortable array
+    names : numpy array
+        Array of region names as stored in the annot file
 
     """
     with open(filepath, "rb") as fobj:
@@ -233,24 +236,174 @@ def read_annot(filepath, orig_ids=False):
     return labels, ctab, names
 
 
-def read_label(filepath):
+def read_label(filepath, read_scalars=False):
     """Load in a Freesurfer .label file.
 
     Parameters
     ----------
     filepath : str
         Path to label file
+    read_scalars : bool
+        If true, read and return scalars associated with each vertex
 
     Returns
     -------
-    label_array : numpy array
+    label_array : numpy array (ints)
         Array with indices of vertices included in label
+    scalar_array : numpy array (floats)
+        If read_scalars is True, array of scalar data for each vertex
 
     """
     label_array = np.loadtxt(filepath, dtype=np.int, skiprows=2, usecols=[0])
+    if read_scalars:
+        scalar_array = np.loadtxt(filepath, skiprows=2, usecols=[-1])
+        return label_array, scalar_array
     return label_array
 
 
+def read_stc(filepath):
+    """Read an STC file from the MNE package
+
+    STC files contain activations or source reconstructions
+    obtained from EEG and MEG data.
+
+    Parameters
+    ----------
+    filepath: string
+        Path to STC file
+
+    Returns
+    -------
+    data: dict
+        The STC structure. It has the following keys:
+           tmin           The first time point of the data in seconds
+           tstep          Time between frames in seconds
+           vertices       vertex indices (0 based)
+           data           The data matrix (nvert * ntime)
+    """
+    fid = open(filepath, 'rb')
+
+    stc = dict()
+
+    fid.seek(0, 2)  # go to end of file
+    file_length = fid.tell()
+    fid.seek(0, 0)  # go to beginning of file
+
+    # read tmin in ms
+    stc['tmin'] = float(np.fromfile(fid, dtype=">f4", count=1))
+    stc['tmin'] /= 1000.0
+
+    # read sampling rate in ms
+    stc['tstep'] = float(np.fromfile(fid, dtype=">f4", count=1))
+    stc['tstep'] /= 1000.0
+
+    # read number of vertices/sources
+    vertices_n = int(np.fromfile(fid, dtype=">I4", count=1))
+
+    # read the source vector
+    stc['vertices'] = np.fromfile(fid, dtype=">I4", count=vertices_n)
+
+    # read the number of timepts
+    data_n = int(np.fromfile(fid, dtype=">I4", count=1))
+
+    if ((file_length / 4 - 4 - vertices_n) % (data_n * vertices_n)) != 0:
+        raise ValueError('incorrect stc file size')
+
+    # read the data matrix
+    stc['data'] = np.fromfile(fid, dtype=">f4", count=vertices_n * data_n)
+    stc['data'] = stc['data'].reshape([data_n, vertices_n]).T
+
+    # close the file
+    fid.close()
+    return stc
+
+
+def project_volume_data(filepath, hemi, reg_file=None, subject_id=None,
+                        projmeth="frac", projsum="avg", projarg=[0, 1, .1],
+                        surf="white", smooth_fwhm=3, mask_label=None,
+                        target_subject=None, verbose=False):
+    """Sample MRI volume onto cortical manifold.
+
+    Note: this requires Freesurfer to be installed with correct
+    SUBJECTS_DIR definition (it uses mri_vol2surf internally).
+
+    Parameters
+    ----------
+    filepath : string
+        Volume file to resample (equivalent to --mov)
+    hemi : [lh, rh]
+        Hemisphere target
+    reg_file : string
+        Path to TKreg style affine matrix file
+    subject_id : string
+        Use if file is in register with subject's orig.mgz
+    projmeth : [frac, dist]
+        Projection arg should be understood as fraction of cortical
+        thickness or as an absolute distance (in mm)
+    projsum : [avg, max, point]
+        Average over projection samples, take max, or take point sample
+    projarg : single float or sequence of three floats
+        Single float for point sample, sequence for avg/max specifying
+        start, stop, and stop
+    surf : string
+        Target surface
+    smooth_fwhm : float
+        FWHM of surface-based smoothing to apply; 0 skips smoothing
+    mask_label : string
+        Path to label file to constrain projection; otherwise uses cortex
+    target_subject : string
+        Subject to warp data to in surface space after projection
+    verbose : boolean
+        If true, print command line
+
+    """
+
+    # Set the basic commands
+    cmd_list = ["mri_vol2surf",
+                "--mov", filepath,
+                "--hemi", hemi,
+                "--surf", surf]
+
+    # Specify the affine registration
+    if reg_file is not None:
+        cmd_list.extend(["--reg", reg_file])
+    elif subject_id is not None:
+        cmd_list.extend(["--regheader", subject_id])
+    else:
+        raise ValueError("Must specify reg_file or subject_id")
+
+    # Specify the projection
+    proj_flag = "--proj" + projmeth
+    if projsum != "point":
+        proj_flag += "-"
+        proj_flag += projsum
+    if hasattr(projarg, "__iter__"):
+        proj_arg = "%.3f, %.3f, %.3f" % tuple(projarg)
+    else:
+        proj_arg = str(projarg)
+    cmd_list.extend([proj_flag, proj_arg])
+
+    # Set misc args
+    if smooth_fwhm:
+        cmd_list.extend(["--surf-fwhm", str(smooth_fwhm)])
+    if mask_label is not None:
+        cmd_list.extend(["--mask", mask_label])
+    if target_subject is not None:
+        cmd_list.extend(["--trgsubject", target_subject])
+
+    # Execute the command
+    out_file = ".tmp-pysurfer.mgz"
+    cmd_list.extend(["--o", out_file])
+    if verbose:
+        print " ".join(cmd_list)
+    out = check_output(cmd_list)
+
+    # Read in the data
+    surf_data = read_scalar_data(out_file)
+    os.remove(out_file)
+    return surf_data
+
+
 class Surface(object):
     """Container for surface object
 
@@ -293,8 +446,8 @@ class Surface(object):
         self.surf = surf
 
         if 'SUBJECTS_DIR' not in os.environ:
-            raise ValueError('Test suite relies on the definition of the '
-                             'SUBJECTS_DIR environment variable')
+            raise ValueError('Surface relies on the definition of the '
+                             'of the SUBJECTS_DIR environment variable')
 
         subj_dir = os.environ["SUBJECTS_DIR"]
         self.data_path = pjoin(subj_dir, subject_id)
@@ -317,7 +470,7 @@ class Surface(object):
         return self.coords[:, 2]
 
     def load_curvature(self):
-        """Load in curavature values from the ?h.curv file."""
+        """Load in curvature values from the ?h.curv file."""
         curv_path = pjoin(self.data_path, "surf", "%s.curv" % self.hemi)
         self.curv = read_morph_data(curv_path)
         self.bin_curv = np.array(self.curv > 0, np.int)
