@@ -5,12 +5,25 @@ from warnings import warn
 import numpy as np
 from scipy import stats
 from scipy import ndimage
+from matplotlib.colors import colorConverter
 
 from . import io
 from . import utils
 from .io import Surface
 from .config import config
 
+try:
+    from traits.api import (HasTraits, Range, Int, Float,
+                            Bool, Enum, on_trait_change)
+except ImportError:
+    from enthought.traits.api import (HasTraits, Range, Int, Float, \
+                                      Bool, Enum, on_trait_change)
+
+try:
+    from traits.ui.api import View, Item, VSplit, HSplit, Group
+except ImportError:
+    from enthought.traits.ui.api import View, Item, VSplit, HSplit, Group
+
 lh_viewdict = {'lateral': {'v': (180., 90.), 'r': 90.},
                 'medial': {'v': (0., 90.), 'r': -90.},
                 'rostral': {'v': (90., 90.), 'r': -180.},
@@ -66,16 +79,18 @@ class Brain(object):
             self.viewdict = rh_viewdict
         self.surf = surf
 
-        # Initialize an mlab figure
-        bg_color_code, size = self.__get_scene_properties(config_opts)
+        # Initialize the mlab figure
         if title is None:
             title = subject_id
+        self._set_scene_properties(config_opts)
         self._f = mlab.figure(title,
-                              bgcolor=bg_color_code,
-                              size=size)
+                              **self.scene_properties)
         mlab.clf()
         self._f.scene.disable_render = True
 
+        # Set the lights so they are oriented by hemisphere
+        self._orient_lights()
+
         # Initialize a Surface object as the geometry
         self._geo = Surface(subject_id, hemi, surf)
 
@@ -96,7 +111,7 @@ class Brain(object):
 
         # mlab surface for the geometry
         if curv:
-            colormap, vmin, vmax, reverse = self.__get_geo_colors(config_opts)
+            colormap, vmin, vmax, reverse = self._get_geo_colors(config_opts)
             self._geo_surf = mlab.pipeline.surface(self._geo_mesh,
                                 colormap=colormap, vmin=vmin, vmax=vmax)
             if reverse:
@@ -111,6 +126,7 @@ class Brain(object):
         self.overlays = dict()
         self.labels = dict()
         self.foci = dict()
+        self.texts = dict()
 
         # Bring up the lateral view
         self.show_view(config.get("visual", "default_view"))
@@ -175,30 +191,28 @@ class Brain(object):
             mlab.roll(roll)
         return mlab.view(), mlab.roll()
 
-    def add_overlay(self, source, min=None, max=None, sign="abs",
-                    name=None, visible=True):
-        """Add an overlay to the overlay dict from a file or array.
+    def _read_scalar_data(self, source, name=None, cast=True):
+        """Load in scalar data from an image stored in a file or an array
 
         Parameters
         ----------
-        src : str or numpy array
-            path to the overlay file or numpy array with data
-        min : float
-            threshold for overlay display
-        max : float
-            saturation point for overlay display
-        sign : {'abs' | 'pos' | 'neg'}
-            whether positive, negative, or both values should be displayed
-        name : str
+        source : str or numpy array
+            path to scalar data file or a numpy array
+        name : str or None, optional
             name for the overlay in the internal dictionary
-        visible : boolean
-            whether the overlay should be visible upon load
+        cast : bool, optional
+            either to cast float data into 64bit datatype as a
+            workaround. cast=True can fix a rendering problem with
+            certain versions of Mayavi
 
+        Returns
+        -------
+        scalar_data : numpy array
+            flat numpy array of scalar data
+        name : str
+            if no name was provided, deduces the name if filename was given
+            as a source
         """
-        try:
-            from mayavi import mlab
-        except ImportError:
-            from enthought.mayavi import mlab
 
         # If source is a string, try to load a file
         if isinstance(source, basestring):
@@ -214,6 +228,39 @@ class Brain(object):
             # Can't think of a good way to check that this will work nicely
             scalar_data = source
 
+        if cast:
+            if (scalar_data.dtype.char == 'f' and
+                scalar_data.dtype.itemsize < 8):
+                scalar_data = scalar_data.astype(np.float)
+
+        return scalar_data, name
+
+    def add_overlay(self, source, min=None, max=None, sign="abs", name=None):
+        """Add an overlay to the overlay dict from a file or array.
+
+        Parameters
+        ----------
+        source : str or numpy array
+            path to the overlay file or numpy array with data
+        min : float
+            threshold for overlay display
+        max : float
+            saturation point for overlay display
+        sign : {'abs' | 'pos' | 'neg'}
+            whether positive, negative, or both values should be displayed
+        name : str
+            name for the overlay in the internal dictionary
+
+        """
+        try:
+            from mayavi import mlab
+        except ImportError:
+            from enthought.mayavi import mlab
+
+        scalar_data, name = self._read_scalar_data(source, name)
+
+        min, max = self._get_display_range(scalar_data, min, max, sign)
+
         if name in self.overlays:
             "%s%d" % (name, len(self.overlays) + 1)
 
@@ -223,12 +270,29 @@ class Brain(object):
         self._f.scene.disable_render = True
         view = mlab.view()
         self.overlays[name] = Overlay(scalar_data, self._geo, min, max, sign)
+        for bar in ["pos_bar", "neg_bar"]:
+            try:
+                self._format_cbar_text(getattr(self.overlays[name], bar))
+            except AttributeError:
+                pass
+
         mlab.view(*view)
         self._f.scene.disable_render = False
 
-    def add_data(self, array, min=None, max=None, colormap="blue-red"):
+    def add_data(self, array, min=None, max=None, thresh=None,
+                 colormap="blue-red", alpha=1,
+                 vertices=None, smoothing_steps=20, time=None,
+                 time_label="time index=%d"):
         """Display data from a numpy array on the surface.
 
+        This provides a similar interface to add_overlay, but it displays
+        it with a single colormap. It offers more flexibility over the
+        colormap, and provides a way to display four dimensional data
+        (i.e. a timecourse).
+
+        Note that min sets the low end of the colormap, and is separate
+        from thresh (this is a different convention from add_overlay)
+
         Parameters
         ----------
         array : numpy array
@@ -237,9 +301,21 @@ class Brain(object):
             min value in colormap (uses real min if None)
         max : float
             max value in colormap (uses real max if None)
+        thresh : None or float
+            if not None, values below thresh will not be visible
         colormap : str
             name of Mayavi colormap to use
-
+        alpha : float in [0, 1]
+            alpha level to control opacity
+        vertices : numpy array
+            vertices for which the data is defined (needed if len(data) < nvtx)
+        smoothing_steps : int
+            number of smoothing steps (smooting is used if len(data) < nvtx)
+            Default : 20
+        time : numpy array
+            time points in the data array (if data is 2D)
+        time_label : str
+            format of the time label
         """
         try:
             from mayavi import mlab
@@ -259,25 +335,81 @@ class Brain(object):
         if max is None:
             max = array.max()
 
+        # Create smoothing matrix if necessary
+        if len(array) < self._geo.x.shape[0]:
+            if vertices == None:
+                raise ValueError("len(data) < nvtx: need vertices")
+            adj_mat = utils.mesh_edges(self._geo.faces)
+            smooth_mat = utils.smoothing_matrix(vertices, adj_mat,
+                                                smoothing_steps)
+        else:
+            smooth_mat = None
+
+        # Calculate initial data to plot
+        if array.ndim == 1:
+            array_plot = array
+        elif array.ndim == 2:
+            array_plot = array[:, 0]
+        else:
+            raise ValueError("data has to be 1D or 2D")
+
+        if smooth_mat != None:
+            array_plot = smooth_mat * array_plot
+
+        # Copy and byteswap to deal with Mayavi bug
+        if array_plot.dtype.byteorder == '>':
+            mlab_plot = array_plot.copy()
+            mlab_plot.byteswap(True)
+        else:
+            mlab_plot = array_plot
+
         # Set up the visualization pipeline
         mesh = mlab.pipeline.triangular_mesh_source(self._geo.x,
                                                     self._geo.y,
                                                     self._geo.z,
                                                     self._geo.faces,
-                                                    scalars=array)
+                                                    scalars=mlab_plot)
+        if thresh is not None:
+            if array_plot.min() >= thresh:
+                warn("Data min is greater than threshold.")
+            else:
+                mesh = mlab.pipeline.threshold(mesh, low=thresh)
         surf = mlab.pipeline.surface(mesh, colormap=colormap,
-                                     vmin=min, vmax=max)
+                                     vmin=min, vmax=max,
+                                     opacity=float(alpha))
 
         # Get the colorbar
         bar = mlab.scalarbar(surf)
+        self._format_cbar_text(bar)
         bar.scalar_bar_representation.position2 = .8, 0.09
 
-        # Fil in the data dict
-        self.data = dict(surface=surf, colorbar=bar)
+        # Get the original colormap table
+        orig_ctable = \
+            surf.module_manager.scalar_lut_manager.lut.table.to_array().copy()
+
+        # Fill in the data dict
+        self.data = dict(surface=surf, colorbar=bar, orig_ctable=orig_ctable,
+                         array=array, smoothing_steps=smoothing_steps,
+                         fmin=min, fmid=(min + max) / 2, fmax=max,
+                         transparent=False, time=0, time_idx=0)
+        if vertices != None:
+            self.data["vertices"] = vertices
+            self.data["smooth_mat"] = smooth_mat
+
         mlab.view(*view)
+
+        # Create time array and add label if 2D
+        if array.ndim == 2:
+            if time == None:
+                time = np.arange(array.shape[1])
+            self.data["time_label"] = time_label
+            self.data["time"] = time
+            self.data["time_idx"] = 0
+            self.add_text(0.05, 0.1, time_label % time[0], name="time_label")
+
         self._f.scene.disable_render = False
 
-    def add_annotation(self, annot, borders=True):
+    def add_annotation(self, annot, borders=True, alpha=1):
         """Add an annotation file.
 
         Parameters
@@ -286,6 +418,8 @@ class Brain(object):
             Either path to annotation file or annotation name
         borders : bool
             Show only borders of regions
+        alpha : float in [0, 1]
+            Alpha level to control opacity
 
         """
         try:
@@ -334,6 +468,10 @@ class Brain(object):
         ids = ord[np.searchsorted(cmap[ord, -1], labels)]
         cmap = cmap[:, :4]
 
+        #  Set the alpha level
+        alpha_vec = cmap[:, 3]
+        alpha_vec[alpha_vec > 0] = alpha * 255
+
         # Maybe get rid of old annot
         if hasattr(self, "annot"):
             self.annot['surface'].remove()
@@ -355,17 +493,24 @@ class Brain(object):
         mlab.view(*view)
         self._f.scene.disable_render = False
 
-    def add_label(self, label, borders=True, color=(76, 169, 117, 255)):
+    def add_label(self, label, color="crimson", alpha=1,
+                  scalar_thresh=None, borders=False):
         """Add an ROI label to the image.
 
         Parameters
         ----------
         label : str
             label filepath or name
+        color : matplotlib-style color
+            anything matplotlib accepts: string, RGB, hex, etc.
+        alpha : float in [0, 1]
+            alpha level to control opacity
+        scalar_thresh : None or number
+            threshold the label ids using this value in the label
+            file's scalar field (i.e. label only vertices with
+            scalar >= thresh)
         borders : bool
             show only label borders
-        color : (float, float, float, float)
-            RGBA color tuple
 
         """
         try:
@@ -390,7 +535,12 @@ class Brain(object):
                 raise ValueError('Label file %s does not exist'
                                  % filepath)
 
-        ids = (io.read_label(filepath),)
+        # Load the label data and create binary overlay
+        if scalar_thresh is None:
+            ids = io.read_label(filepath)
+        else:
+            ids, scalars = io.read_label(filepath, read_scalars=True)
+            ids = ids[scalars >= scalar_thresh]
         label = np.zeros(self._geo.coords.shape[0])
         label[ids] = 1
 
@@ -409,9 +559,8 @@ class Brain(object):
                                                    scalars=label)
         surf = mlab.pipeline.surface(mesh, name=label_name)
 
-        if not isinstance(color, tuple) or len(color) != 4:
-            raise TypeError("'color' parameter must be a 4-tuple")
-        cmap = np.array([(0, 0, 0, 0,), color])
+        color = colorConverter.to_rgba(color, alpha)
+        cmap = np.array([(0, 0, 0, 0,), color]) * 255
         surf.module_manager.scalar_lut_manager.lut.table = cmap
 
         self.labels[label_name] = surf
@@ -490,6 +639,7 @@ class Brain(object):
 
         # Get the colorbar
         bar = mlab.scalarbar(surf)
+        self._format_cbar_text(bar)
         bar.scalar_bar_representation.position2 = .8, 0.09
 
         # Fil in the morphometry dict
@@ -500,7 +650,7 @@ class Brain(object):
         self._f.scene.disable_render = False
 
     def add_foci(self, coords, coords_as_verts=False, map_surface=None,
-                 scale_factor=1, color=(1, 1, 1), name=None):
+                 scale_factor=1, color="white", alpha=1, name=None):
         """Add spherical foci, possibly mapping to displayed surf.
 
         The foci spheres can be displayed at the coordinates given, or
@@ -519,8 +669,10 @@ class Brain(object):
             surface to map coordinates through, or None to use raw coords
         scale_factor : int
             controls the size of the foci spheres
-        color : 3-tuple
-            RGB color code for foci spheres
+        color : matplotlib color code
+            HTML name, RBG tuple, or hex code
+        alpha : float in [0, 1]
+            opacity of focus gylphs
         name : str
             internal name to use
 
@@ -548,6 +700,9 @@ class Brain(object):
         if name is None:
             name = "foci_%d" % (len(self.foci) + 1)
 
+        # Convert the color code
+        color = colorConverter.to_rgb(color)
+
         # Create the visualization
         self._f.scene.disable_render = True
         view = mlab.view()
@@ -555,23 +710,23 @@ class Brain(object):
                                foci_coords[:, 1],
                                foci_coords[:, 2],
                                np.ones(foci_coords.shape[0]),
-                               scale_factor=(5. * scale_factor),
-                               color=color, name=name)
+                               scale_factor=(10. * scale_factor),
+                               color=color, opacity=alpha, name=name)
         self.foci[name] = points
         mlab.view(*view)
         self._f.scene.disable_render = False
 
-    def add_contour_overlay(self, filepath, min=None, max=None,
+    def add_contour_overlay(self, source, min=None, max=None,
                             n_contours=7, line_width=1.5):
-        """Add a topographic contour overlay.
+        """Add a topographic contour overlay of the positive data.
 
         Note: This visualization will look best when using the "low_contrast"
         cortical curvature colorscheme.
 
         Parameters
         ----------
-        filepath : str
-            path to the overlay file (must be readable by Nibabel, or .mgh)
+        source : str or array
+            path to the overlay file or numpy array
         min : float
             threshold for overlay display
         max : float
@@ -588,39 +743,9 @@ class Brain(object):
             from enthought.mayavi import mlab
 
         # Read the scalar data
-        scalar_data = io.read_scalar_data(filepath)
+        scalar_data, _ = self._read_scalar_data(source)
 
-        #TODO find a better place for this; duplicates code in Overlay object
-        if min is None:
-            try:
-                min = config.getfloat("overlay", "min_thresh")
-            except ValueError:
-                min_str = config.get("overlay", "min_thresh")
-                if min_str == "robust_min":
-                    min = stats.scoreatpercentile(scalar_data, 2)
-                elif min_str == "actual_min":
-                    min = scalar_data.min()
-                else:
-                    min = 2.0
-                    warn("The 'min_thresh' value in your config value must be "
-                "a float, 'robust_min', or 'actual_min', but it is %s. "
-                "I'm setting the overlay min to the config default of 2" % min)
-
-        if max is None:
-            try:
-                max = config.getfloat("overlay", "max_thresh")
-            except ValueError:
-                max_str = config.get("overlay", "max_thresh")
-                if max_str == "robust_max":
-                    max = stats.scoreatpercentile(scalar_data, 98)
-                elif max_str == "actual_max":
-                    max = scalar_data.max()
-                else:
-                    max = stats.scoreatpercentile(scalar_data, 98)
-                    warn("The 'max_thresh' value in your config value must be "
-                "a float, 'robust_min', or 'actual_min', but it is %s. "
-                "I'm setting the overlay min to the config default "
-                "of robust_max" % max)
+        min, max = self._get_display_range(scalar_data, min, max, "pos")
 
         # Prep the viz
         self._f.scene.disable_render = True
@@ -644,8 +769,11 @@ class Brain(object):
                                              line_width=line_width)
 
         # Set the colorbar and range correctly
-        bar = mlab.scalarbar(surf)
+        bar = mlab.scalarbar(surf,
+                             nb_colors=n_contours,
+                             nb_labels=n_contours + 1)
         bar.data_range = min, max
+        self._format_cbar_text(bar)
         bar.scalar_bar_representation.position2 = .8, 0.09
 
         # Set up a dict attribute with pointers at important things
@@ -655,35 +783,61 @@ class Brain(object):
         mlab.view(*view)
         self._f.scene.disable_render = False
 
-    def __get_scene_properties(self, config_opts):
-        """Get the background color and size from the config parser.
+    def add_text(self, x, y, text, name, color=(1, 1, 1), opacity=1.0):
+        """ Add a text to the visualization
+
+        Parameters
+        ----------
+        x : Float
+            x coordinate
+        y : Float
+            y coordinate
+        text : str
+            Text to add
+        name : str
+            Name of the text (text label can be updated using update_text())
+        color : Tuple
+            Color of the text. Default: (1, 1, 1)
+        opacity : Float
+            Opacity of the text. Default: 1.0
+        """
+        try:
+            from mayavi import mlab
+        except ImportError:
+            from enthought.mayavi import mlab
+
+        text = mlab.text(x, y, text, figure=None, name=name,
+                         color=color, opacity=opacity)
+
+        self.texts[name] = text
+
+    def _set_scene_properties(self, config_opts):
+        """Set the scene_prop dict from the config parser.
 
         Parameters
         ----------
         config_opts : dict
             dictionary of config file "visual" options
 
-        Returns
-        -------
-        bg_color_code : (float, float, float)
-            RBG code for background color
-        size : (float, float)
-            viewer window size
-
         """
-        bg_colors = dict(black=[0, 0, 0],
-                         white=[256, 256, 256],
-                         midnight=[12, 7, 32],
-                         slate=[112, 128, 144],
-                         charcoal=[59, 69, 79],
-                         sand=[245, 222, 179])
+        colors = dict(black=[0, 0, 0],
+                      white=[256, 256, 256],
+                      midnight=[12, 7, 32],
+                      slate=[112, 128, 144],
+                      charcoal=[59, 69, 79],
+                      sand=[245, 222, 179])
 
         try:
             bg_color_name = config_opts['background']
         except KeyError:
             bg_color_name = config.get("visual", "background")
-        bg_color_code = bg_colors[bg_color_name]
-        bg_color_code = tuple(map(lambda x: float(x) / 256, bg_color_code))
+        bg_color_code = colorConverter.to_rgb(bg_color_name)
+
+        try:
+            fg_color_name = config_opts['foreground']
+        except KeyError:
+            fg_color_name = config.get("visual", "foreground")
+        fg_color_code = colorConverter.to_rgb(fg_color_name)
 
         try:
             size = config_opts['size']
@@ -691,9 +845,17 @@ class Brain(object):
             size = config.getfloat("visual", "size")
         size = (size, size)
 
-        return bg_color_code, size
+        self.scene_properties = dict(fgcolor=fg_color_code,
+                                     bgcolor=bg_color_code,
+                                     size=size)
 
-    def __get_geo_colors(self, config_opts):
+    def _orient_lights(self):
+        """Set lights to come from same direction relative to brain."""
+        if self.hemi == "rh":
+            for light in self._f.scene.light_manager.lights:
+                light.azimuth *= -1
+
+    def _get_geo_colors(self, config_opts):
         """Return an mlab colormap name, vmin, and vmax for binary curvature.
 
         Parameters
@@ -734,6 +896,43 @@ class Brain(object):
 
         return color_data
 
+    def get_data_properties(self):
+        """ Get properties of the data shown
+
+        Returns
+        -------
+        props : dict
+            Dictionary with data properties
+
+            props["fmin"] : minimum colormap
+            props["fmid"] : midpoint colormap
+            props["fmax"] : maximum colormap
+            props["transparent"] : lower part of colormap transparent?
+            props["time"] : time points
+            props["time_idx"] : current time index
+            props["smoothing_steps"] : number of smoothing steps
+        """
+        props = dict()
+        try:
+            props["fmin"] = self.data["fmin"]
+            props["fmid"] = self.data["fmid"]
+            props["fmax"] = self.data["fmax"]
+            props["transparent"] = self.data["transparent"]
+            props["time"] = self.data["time"]
+            props["time_idx"] = self.data["time_idx"]
+            props["smoothing_steps"] = self.data["smoothing_steps"]
+        except KeyError:
+            # The user has not added any data
+            props["fmin"] = 0
+            props["fmid"] = 0
+            props["fmax"] = 0
+            props["transparent"] = 0
+            props["time"] = 0
+            props["time_idx"] = 0
+            props["smoothing_steps"] = 0
+
+        return props
+
     def save_image(self, fname):
         """Save current view to disk
 
@@ -795,6 +994,112 @@ class Brain(object):
                 print("Skipping %s: not in view dict" % view)
         return images_written
 
+    def save_image_sequence(self, time_idx, fname_pattern, use_abs_idx=True):
+        """Save a temporal image sequence
+
+        The files saved are named "fname_pattern % (pos)" where "pos" is a
+        relative or absolute index (controlled by "use_abs_idx")
+
+        Parameters
+        ----------
+        time_idx : array-like
+            time indices to save
+        fname_pattern : str
+            filename pattern, e.g. 'movie-frame_%0.4d.png'
+        use_abs_idx : boolean
+            if True the indices given by "time_idx" are used in the filename
+            if False the index in the filename starts at zero and is
+            incremented by one for each image (Default: True)
+
+        Returns
+        -------
+        images_written: list
+            all filenames written
+        """
+
+        current_time_idx = self.data["time_idx"]
+
+        images_written = list()
+        rel_pos = 0
+        for idx in time_idx:
+            self.set_data_time_index(idx)
+            fname = fname_pattern % (idx if use_abs_idx else rel_pos)
+            self.save_image(fname)
+            images_written.append(fname)
+            rel_pos += 1
+
+        # Restore original time index
+        self.set_data_time_index(current_time_idx)
+
+        return images_written
+
+    def scale_data_colormap(self, fmin, fmid, fmax, transparent):
+        """Scale the data colormap.
+
+        Parameters
+        ----------
+        fmin : float
+            minimum value of colormap
+        fmid : float
+            value corresponding to color midpoint
+        fmax : float
+            maximum value for colormap
+        transparent : boolean
+            if True: use a linear transparency between fmin and fmid
+        """
+
+        if not (fmin < fmid) and (fmid < fmax):
+            raise ValueError("Invalid colormap, we need fmin<fmid<fmax")
+
+        # Cast inputs to float to prevent integer division
+        fmin = float(fmin)
+        fmid = float(fmid)
+        fmax = float(fmax)
+
+        print "colormap: fmin=%0.2e fmid=%0.2e fmax=%0.2e transparent=%d" \
+              % (fmin, fmid, fmax, transparent)
+
+        # Get the original colormap
+        table = self.data["orig_ctable"].copy()
+
+        # Add transparency if needed
+        if  transparent:
+            n_colors = table.shape[0]
+            n_colors2 = int(n_colors / 2)
+            table[:n_colors2, -1] = np.linspace(0, 255, n_colors2)
+            table[n_colors2:, -1] = 255 * np.ones(n_colors - n_colors2)
+
+        # Scale the colormap
+        table_new = table.copy()
+        n_colors = table.shape[0]
+        n_colors2 = int(n_colors / 2)
+
+        # Index of fmid in new colorbar
+        fmid_idx = np.round(n_colors * ((fmid - fmin) / (fmax - fmin))) - 1
+
+        # Go through channels
+        for i in range(4):
+            part1 = np.interp(np.linspace(0, n_colors2 - 1, fmid_idx + 1),
+                              np.arange(n_colors),
+                              table[:, i])
+            table_new[:fmid_idx + 1, i] = part1
+            part2 = np.interp(np.linspace(n_colors2, n_colors - 1,
+                                          n_colors - fmid_idx - 1),
+                              np.arange(n_colors),
+                              table[:, i])
+            table_new[fmid_idx + 1:, i] = part2
+
+        # Get the new colormap
+        cmap = self.data["surface"].module_manager.scalar_lut_manager
+        cmap.lut.table = table_new
+        cmap.data_range = np.array([fmin, fmax])
+
+        # Update the data properties
+        self.data["fmin"] = fmin
+        self.data["fmid"] = fmid
+        self.data["fmax"] = fmax
+        self.data["transparent"] = transparent
+
     def save_montage(self, filename, order=['lat', 'ven', 'med'],
                      orientation='h', border_size=15):
         """Create a montage from a given order of images
@@ -863,6 +1168,68 @@ class Brain(object):
         for f in fnames:
             os.remove(f)
 
+    def set_data_time_index(self, time_idx):
+        """ Set the data time index to show
+
+        Parameters
+        ----------
+        time_idx : int
+            time index
+        """
+        if time_idx < 0 or time_idx >= self.data["array"].shape[1]:
+            raise ValueError("time index out of range")
+
+        plot_data = self.data["array"][:, time_idx]
+
+        if "smooth_mat" in self.data:
+            plot_data = self.data["smooth_mat"] * plot_data
+        self.data["surface"].mlab_source.scalars = plot_data
+        self.data["time_idx"] = time_idx
+
+        # Update time label
+        self.update_text(self.data["time_label"] % self.data["time"][time_idx],
+                         "time_label")
+
+    def set_data_smoothing_steps(self, smoothing_steps):
+        """ Set the number of smoothing steps
+
+        Parameters
+        ----------
+        smoothing_steps : int
+            Number of smoothing steps
+        """
+
+        adj_mat = utils.mesh_edges(self._geo.faces)
+        smooth_mat = utils.smoothing_matrix(self.data["vertices"], adj_mat,
+                                            smoothing_steps)
+
+        self.data["smooth_mat"] = smooth_mat
+
+        # Redraw
+        if self.data["array"].ndim == 1:
+            plot_data = self.data["array"]
+        else:
+            plot_data = self.data["array"][:, self.data["time_idx"]]
+
+        plot_data = self.data["smooth_mat"] * plot_data
+
+        self.data["surface"].mlab_source.scalars = plot_data
+
+        # Update data properties
+        self.data["smoothing_steps"] = smoothing_steps
+
+    def update_text(self, text, name):
+        """ Update text label
+
+        Parameters
+        ----------
+        text : str
+            New text for label
+        name : str
+            Name of text label
+        """
+        self.texts[name].text = text
+
     def min_diff(self, beg, end):
         """Determine minimum "camera distance" between two views.
 
@@ -1001,16 +1368,7 @@ class Brain(object):
         mlab.close(self._f)
         #should we tear down other variables?
 
-
-class Overlay(object):
-
-    def __init__(self, scalar_data, geo, min, max, sign):
-        """
-        """
-        try:
-            from mayavi import mlab
-        except ImportError:
-            from enthought.mayavi import mlab
+    def _get_display_range(self, scalar_data, min, max, sign):
 
         if scalar_data.min() >= 0:
             sign = "pos"
@@ -1026,6 +1384,7 @@ class Overlay(object):
         else:
             range_data = np.abs(scalar_data)
 
+        # Get the min and max from among various places
         if min is None:
             try:
                 min = config.getfloat("overlay", "min_thresh")
@@ -1047,7 +1406,7 @@ class Overlay(object):
             except ValueError:
                 max_str = config.get("overlay", "max_thresh")
                 if max_str == "robust_max":
-                    max = stats.scoreatpercentile(range_data, 98)
+                    max = stats.scoreatpercentile(scalar_data, 98)
                 elif max_str == "actual_max":
                     max = range_data.max()
                 else:
@@ -1057,10 +1416,31 @@ class Overlay(object):
                 "I'm setting the overlay min to the config default "
                 "of robust_max" % max)
 
-        # Clean up range_data since we don't need it and it might be big
-        del range_data
+        return min, max
 
-        # Byte swap inplace; due to mayavi bug
+    def _format_cbar_text(self, cbar):
+
+        bg_color = self.scene_properties["bgcolor"]
+        text_color = (1., 1., 1.) if sum(bg_color) < 2 else (0., 0., 0.)
+        cbar.label_text_property.color = text_color
+
+
+class Overlay(object):
+    """Encapsulation of statistical neuroimaging overlay visualization."""
+
+    def __init__(self, scalar_data, geo, min, max, sign):
+        try:
+            from mayavi import mlab
+        except ImportError:
+            from enthought.mayavi import mlab
+
+        if scalar_data.min() >= 0:
+            sign = "pos"
+        elif scalar_data.max() <= 0:
+            sign = "neg"
+        self.sign = sign
+
+        # Byte swap copy; due to mayavi bug
         mlab_data = scalar_data.copy()
         if scalar_data.dtype.byteorder == '>':
             mlab_data.byteswap(True)
@@ -1086,7 +1466,7 @@ class Overlay(object):
             pos_thresh = mlab.pipeline.threshold(pos_mesh, low=thresh_low)
             pos_surf = mlab.pipeline.surface(pos_thresh, colormap="YlOrRd",
                                              vmin=min, vmax=max)
-            pos_bar = mlab.scalarbar(pos_surf)
+            pos_bar = mlab.scalarbar(pos_surf, nb_labels=5)
             pos_bar.reverse_lut = True
 
             self.pos = pos_surf
@@ -1113,21 +1493,12 @@ class Overlay(object):
             neg_thresh = mlab.pipeline.threshold(neg_mesh, up=thresh_up)
             neg_surf = mlab.pipeline.surface(neg_thresh, colormap="PuBu",
                                              vmin=-max, vmax=-min)
-            neg_bar = mlab.scalarbar(neg_surf)
+            neg_bar = mlab.scalarbar(neg_surf, nb_labels=5)
 
             self.neg = neg_surf
             self.neg_bar = neg_bar
 
-        self.__format_colorbar()
-
-    def toggle_visibility(self):
-
-        if self.sign in ["pos", "abs"]:
-            self.pos.visible = not self.pos.visible
-            self.pos_bar.visible = False
-        if self.sign in ["neg", "abs"]:
-            self.neg.visible = not self.neg.visible
-            self.neg_bar.visible = False
+        self._format_colorbar()
 
     def remove(self):
 
@@ -1138,7 +1509,7 @@ class Overlay(object):
             self.neg.remove()
             self.neg_bar.visible = False
 
-    def __format_colorbar(self):
+    def _format_colorbar(self):
 
         if self.sign in ["abs", "neg"]:
             self.neg_bar.scalar_bar_representation.position = (0.05, 0.01)
@@ -1146,3 +1517,100 @@ class Overlay(object):
         if self.sign in ["abs", "pos"]:
             self.pos_bar.scalar_bar_representation.position = (0.53, 0.01)
             self.pos_bar.scalar_bar_representation.position2 = (0.42, 0.09)
+
+
+class TimeViewer(HasTraits):
+    """ TimeViewer object providing a GUI for visualizing time series, such
+        as M/EEG inverse solutions, on Brain object(s)
+    """
+    min_time = Int(0)
+    max_time = Int(1E9)
+    current_time = Range(low="min_time", high="max_time", value=0)
+    # colormap: only update when user presses Enter
+    fmax = Float(enter_set=True, auto_set=False)
+    fmid = Float(enter_set=True, auto_set=False)
+    fmin = Float(enter_set=True, auto_set=False)
+    transparent = Bool(True)
+    smoothing_steps = Int(20, enter_set=True, auto_set=False)
+    orientation = Enum("lateral", "medial", "rostral", "caudal",
+                       "dorsal", "ventral", "frontal", "parietal")
+
+    # GUI layout
+    view = View(VSplit(Item(name="current_time"),
+                       Group(HSplit(Item(name="fmin"),
+                                    Item(name="fmid"),
+                                    Item(name="fmax"),
+                                    Item(name="transparent"),
+                                   ),
+                             label="Color scale",
+                             show_border=True
+                            ),
+                        Item(name="smoothing_steps"),
+                        Item(name="orientation")
+                      )
+                )
+
+    def __init__(self, brain):
+        """Initialize TimeViewer
+
+        Parameters
+        ----------
+        brain : Brain
+            brain to control
+        """
+        super(TimeViewer, self).__init__()
+
+        self.brain = brain
+
+        # Initialize GUI with values from brain
+        props = brain.get_data_properties()
+
+        self._disable_updates = True
+        self.max_time = len(props["time"]) - 1
+        self.current_time = props["time_idx"]
+        self.fmin = props["fmin"]
+        self.fmid = props["fmid"]
+        self.fmax = props["fmax"]
+        self.transparent = props["transparent"]
+        self.smoothing_steps = props["smoothing_steps"]
+        self._disable_updates = False
+
+        # Show GUI
+        self.configure_traits()
+
+    @on_trait_change("smoothing_steps")
+    def set_smoothing_steps(self):
+        """ Change number of smooting steps
+        """
+        if self._disable_updates:
+            return
+
+        self.brain.set_data_smoothing_steps(self.smoothing_steps)
+
+    @on_trait_change("orientation")
+    def set_orientation(self):
+        """ Set the orientation
+        """
+        if self._disable_updates:
+            return
+
+        self.brain.show_view(view=self.orientation)
+
+    @on_trait_change("current_time")
+    def set_time_point(self):
+        """ Set the time point shown
+        """
+        if self._disable_updates:
+            return
+
+        self.brain.set_data_time_index(self.current_time)
+
+    @on_trait_change("fmin, fmid, fmax, transparent")
+    def scale_colormap(self):
+        """ Scale the colormap
+        """
+        if self._disable_updates:
+            return
+
+        self.brain.scale_data_colormap(self.fmin, self.fmid, self.fmax,
+                                       self.transparent)
